import { Inject, Injectable } from '@nestjs/common';
import { CreateOrderDto, UpdateOrderDto } from './dto';
import { Order } from './entities/order.entity';
import { ORDER_REPOSITORY } from 'src/core/constants';
import { OrderStatuses } from './entities/order-status.entity';
import { User } from '../users/entities/user.entity';
import { UUID } from 'crypto';

@Injectable()
export class OrdersService {
  constructor(
    @Inject(ORDER_REPOSITORY) private readonly orderRepository: typeof Order,
  ) {}

  async create(createOrderDto: CreateOrderDto) {
    return await this.orderRepository.create(createOrderDto);
  }

  async findAll() {
    return await this.orderRepository.findAll();

    // For joining all the associated tables with the orders table
    // return await this.orderRepository.findAll({
    //   include: [{ model: User }, { model: OrderStatuses }],
    // });
  }

  async findOne(id: number) {
    return await this.orderRepository.findAll({
      where: { id },
    });
  }

  async findByUser(id: UUID) {
    return await this.orderRepository.findAll({
      include: {
        model: User,
        where: { uuid: id },
        attributes: ['firstName', 'lastName'],
      },
    });
  }

  async update(id: number, updateOrderDto: UpdateOrderDto) {
    const [numberOfAffectedRows, [updatedOrder]] =
      await this.orderRepository.update(
        { ...updateOrderDto },
        { where: { id }, returning: true },
      );
    return { numberOfAffectedRows, updatedOrder };
  }

  async remove(id: number) {
    const deletedOrder = await this.findOne(id);

    await this.orderRepository.destroy({ where: { id } });

    return deletedOrder;
  }
}
