import { Inject, Injectable } from '@nestjs/common';
import { CreateShoppingCardDto } from './dto/create-shopping-card.dto';
import { UpdateShoppingCardDto } from './dto/update-shopping-card.dto';
import { SHOPPING_CARD_REPOSITORY } from 'src/core/constants';
import { ShoppingCard } from './entities/shopping-card.entity';
import { CreateReceiptDto } from '../receipts/dto';
import { ReceiptsService } from '../receipts/receipts.service';
import { ProductsService } from '../products/products.service';
import { OrdersService } from '../orders/orders.service';
import { CreateOrderDto } from '../orders/dto';

@Injectable()
export class ShoppingCardsService {
  constructor(
    @Inject(SHOPPING_CARD_REPOSITORY)
    private readonly shoppingCardRepository: typeof ShoppingCard,
    private readonly receiptService: ReceiptsService,
    private readonly productsService: ProductsService,
    private readonly ordersService: OrdersService,
  ) {}
  async create(createShoppingCardDto: CreateShoppingCardDto) {
    const card = await this.shoppingCardRepository.create(
      createShoppingCardDto,
    );

    for (let p of card.products) {
      const [product] = await this.productsService.findOne(p);
      card.totalPrice += product.pricePerUnit;
    }

    return await card.save();
  }

  async findAll() {
    return await this.shoppingCardRepository.findAll();
  }

  async findbyUser(id: number) {
    return await this.shoppingCardRepository.findAll({ where: { userId: id } });
  }

  async update(id: number, updateShoppingCardDto: UpdateShoppingCardDto) {
    const [numberOfAffectedRows, [updatedCard]] =
      await this.shoppingCardRepository.update(
        { ...updateShoppingCardDto },
        { where: { userId: id }, returning: true },
      );

    const [card] = await this.shoppingCardRepository.findAll({
      where: { userId: id },
    });

    card.totalPrice = 0;

    for (let p of card.products) {
      const [product] = await this.productsService.findOne(p);
      card.totalPrice += product.pricePerUnit;
    }

    return card.save();
  }

  async remove(id: number) {
    return await this.shoppingCardRepository.destroy({ where: { userId: id } });
  }

  async submitPurchase(id: number) {
    const [card] = await this.findbyUser(id);

    await this.createReceipt({
      userId: id,
      products: card.products,
      totalPrice: card.totalPrice,
    });

    await this.remove(id);

    const order: CreateOrderDto = {
      userId: id,
      status: 1,
      comments: 'ship asap',
      shipperName: 'meti comp',
      products: card.products,
    };
    return await this.ordersService.create(order);
  }

  private async createReceipt(createReceiptDto: CreateReceiptDto) {
    await this.receiptService.create(createReceiptDto);
  }
}
