import { BadRequestException, Inject, Injectable } from '@nestjs/common';
import { UUID } from 'crypto';
import { WALLET_REPOSITORY } from 'src/core/constants';
import { Wallet } from './entities/wallet.entity';
import { User } from '../users/entities/user.entity';
import { UpdateWalletDto } from './dto/update-wallet.dto';
import { WalletsTransactionsService } from '../wallets-transactions/wallets-transactions.service';
import { UsersService } from '../users/users.service';

@Injectable()
export class WalletsService {
  constructor(
    @Inject(WALLET_REPOSITORY) private readonly walletRepository: typeof Wallet,
    private readonly usersService: UsersService,
    private readonly walletsTransactionsService: WalletsTransactionsService,
  ) {}

  async getWallet(id: UUID) {
    return await this.walletRepository.findAll({
      include: {
        model: User,
        where: { uuid: id },
        attributes: ['firstName'],
      },
    });
  }

  async deposit(id: UUID, updateWalletDto: UpdateWalletDto) {
    const { amount } = updateWalletDto;

    if (amount < 0) throw new BadRequestException();

    const [wallet] = await this.getWallet(id);

    wallet.balance += amount;
    await wallet.save();

    this.createTransaction(id, 1, amount);
  }

  async withdraw(id: UUID, updateWalletDto: UpdateWalletDto) {
    const { amount } = updateWalletDto;

    if (amount < 0) throw new BadRequestException();

    const [wallet] = await this.getWallet(id);

    wallet.balance -= amount;
    await wallet.save();

    this.createTransaction(id, 2, amount);
  }

  private async createTransaction(
    id: UUID,
    operation: number,
    byAmount: number,
  ) {
    const [user] = await this.usersService.findOne(id);

    const transaction = {
      operation: operation,
      byAmount: byAmount,
      userId: user.id,
    };
    await this.walletsTransactionsService.create(transaction);
  }
}
